@@ -4,15 +4,16 @@ declared_s3_generics <- function(x) {
     "/exprlist/expr",
 
     # Assigns to a symbol
-      "[./LEFT_ASSIGN|EQ_ASSIGN]",
-      "[./expr[FUNCTION]]",
-      "[./expr/SYMBOL]",
+    "[./LEFT_ASSIGN|EQ_ASSIGN]",
+    "[./expr[FUNCTION or OP-LAMBDA]]",
+    "[./expr/SYMBOL]",
 
     # Is a S3 Generic (contains call to UseMethod)
-      "[.//SYMBOL_FUNCTION_CALL[text()='UseMethod']]",
+    "[.//SYMBOL_FUNCTION_CALL[text()='UseMethod']]",
 
     # Retrieve assigned name of the function
-    "/expr/SYMBOL/text()")
+    "/expr/SYMBOL/text()"
+  )
 
-  as.character(xml2::xml_find_all(x, xpath))
+  as.character(xml_find_all(x, xpath))
 }
