@@ -46,19 +46,16 @@ equals_na_linter <- function() {
     /parent::expr
   ")
 
-  Linter(function(source_expression) {
-    if (!is_lint_level(source_expression, "expression")) {
-      return(list())
-    }
-
+  Linter(linter_level = "expression", function(source_expression) {
     xml <- source_expression$xml_parsed_content
 
     bad_expr <- xml_find_all(xml, xpath)
+    op <- xml_find_first(bad_expr, "EQ | NE | SPECIAL")
 
     xml_nodes_to_lints(
       bad_expr,
       source_expression,
-      lint_message = "Use is.na for comparisons to NA (not == or != or %in%)",
+      lint_message = sprintf("Use is.na() instead of x %s NA", xml_text(op)),
       type = "warning"
     )
   })
