@@ -43,30 +43,24 @@ expect_type_linter <- function() {
   )
   base_type_tests <- xp_text_in_table(paste0("is.", base_types))
   expect_equal_identical_xpath <- "
-  //SYMBOL_FUNCTION_CALL[text() = 'expect_equal' or text() = 'expect_identical']
-    /parent::expr
-    /following-sibling::expr[
-      expr[1][SYMBOL_FUNCTION_CALL[text() = 'typeof']]
-      and (position() = 1 or preceding-sibling::expr[STR_CONST])
-    ]
+  following-sibling::expr[
+    expr[1][SYMBOL_FUNCTION_CALL[text() = 'typeof']]
+    and (position() = 1 or preceding-sibling::expr[STR_CONST])
+  ]
     /parent::expr[not(SYMBOL_SUB[text() = 'info' or text() = 'label' or text() = 'expected.label'])]
   "
   expect_true_xpath <- glue("
-  //SYMBOL_FUNCTION_CALL[text() = 'expect_true']
-    /parent::expr
-    /following-sibling::expr[1][expr[1][SYMBOL_FUNCTION_CALL[ {base_type_tests} ]]]
+  following-sibling::expr[1][expr[1][SYMBOL_FUNCTION_CALL[ {base_type_tests} ]]]
     /parent::expr[not(SYMBOL_SUB[text() = 'info' or text() = 'label'])]
   ")
-  xpath <- paste(expect_equal_identical_xpath, "|", expect_true_xpath)
 
-  Linter(function(source_expression) {
-    if (!is_lint_level(source_expression, "expression")) {
-      return(list())
-    }
-
-    xml <- source_expression$xml_parsed_content
-
-    bad_expr <- xml_find_all(xml, xpath)
+  Linter(linter_level = "expression", function(source_expression) {
+    expect_equal_identical_calls <- source_expression$xml_find_function_calls(c("expect_equal", "expect_identical"))
+    expect_true_calls <- source_expression$xml_find_function_calls("expect_true")
+    bad_expr <- combine_nodesets(
+      xml_find_all(expect_equal_identical_calls, expect_equal_identical_xpath),
+      xml_find_all(expect_true_calls, expect_true_xpath)
+    )
     matched_function <- xp_call_name(bad_expr)
     msg <- ifelse(
       matched_function %in% c("expect_equal", "expect_identical"),
