@@ -1,10 +1,10 @@
 #' Parsed sourced file from a filename
 #'
-#' This object is given as input to each linter
+#' This object is given as input to each linter.
 #'
 #' @details
-#' The file is read in using the `encoding` setting.
-#' This setting found by taking the first valid result from the following locations
+#' The file is read using the `encoding` setting.
+#' This setting is found by taking the first valid result from the following locations
 #'
 #' 1. The `encoding` key from the usual lintr configuration settings.
 #' 2. The `Encoding` field from a Package `DESCRIPTION` file in a parent directory.
@@ -18,41 +18,48 @@
 #' \describe{
 #'   \item{expressions}{a `list` of
 #'   `n+1` objects. The first `n` elements correspond to each expression in
-#'   `filename`, and consist of a list of 9 elements:
+#'   `filename`, and consist of a list of 8 elements:
 #'   \itemize{
-#'     \item{`filename` (`character`)}
-#'     \item{`line` (`integer`) the line in `filename` where this expression begins}
-#'     \item{`column` (`integer`) the column in `filename` where this expression begins}
+#'     \item{`filename` (`character`) the name of the file.}
+#'     \item{`line` (`integer`) the line in the file where this expression begins.}
+#'     \item{`column` (`integer`) the column in the file where this expression begins.}
 #'     \item{`lines` (named `character`) vector of all lines spanned by this
-#'           expression, named with the line number corresponding to `filename`}
-#'     \item{`parsed_content` (`data.frame`) as given by [utils::getParseData()] for this expression}
-#'     \item{`xml_parsed_content` (`xml_document`) the XML parse tree of this
-#'          expression as given by [xmlparsedata::xml_parse_data()]}
-#'     \item{`content` (`character`) the same as `lines` as a single string (not split across lines)}
-#'     \item{(**Deprecated**) `find_line` (`function`) a function for returning lines in this expression}
-#'     \item{(**Deprecated**) `find_column` (`function`) a similar function for columns}
+#'           expression, named with the corresponding line numbers.}
+#'     \item{`parsed_content` (`data.frame`) as given by [utils::getParseData()] for this expression.}
+#'     \item{`xml_parsed_content` (`xml_document`) the XML parse tree of this expression as given by
+#'           [xmlparsedata::xml_parse_data()].}
+#'     \item{`content` (`character`) the same as `lines` as a single string (not split across lines).}
+#'     \item{`xml_find_function_calls(function_names)` (`function`) a function that returns all `SYMBOL_FUNCTION_CALL`
+#'           XML nodes from `xml_parsed_content` with specified function names.}
 #'   }
 #'
 #'   The final element of `expressions` is a list corresponding to the full file
-#'   consisting of 6 elements:
+#'   consisting of 7 elements:
 #'   \itemize{
-#'     \item{`filename` (`character`)}
-#'     \item{`file_lines` (`character`) the [readLines()] output for this file}
+#'     \item{`filename` (`character`) the name of this file.}
+#'     \item{`file_lines` (`character`) the [readLines()] output for this file.}
 #'     \item{`content` (`character`) for .R files, the same as `file_lines`;
-#'           for .Rmd scripts, this is the extracted R source code (as text)}
+#'           for .Rmd or .qmd scripts, this is the extracted R source code (as text).}
 #'     \item{`full_parsed_content` (`data.frame`) as given by
-#'           [utils::getParseData()] for the full content}
+#'           [utils::getParseData()] for the full content.}
 #'     \item{`full_xml_parsed_content` (`xml_document`) the XML parse tree of all
-#'           expressions as given by [xmlparsedata::xml_parse_data()]}
+#'           expressions as given by [xmlparsedata::xml_parse_data()].}
 #'     \item{`terminal_newline` (`logical`) records whether `filename` has a terminal
-#'           newline (as determined by [readLines()] producing a corresponding warning)}
+#'           newline (as determined by [readLines()] producing a corresponding warning).}
+#'     \item{`xml_find_function_calls(function_names)` (`function`) a function that returns all `SYMBOL_FUNCTION_CALL`
+#'           XML nodes from `full_xml_parsed_content` with specified function names.}
 #'   }
 #'   }
 #'   \item{error}{A `Lint` object describing any parsing error.}
 #'   \item{lines}{The [readLines()] output for this file.}
 #' }
+#'
+#' @examples
+#' tmp <- tempfile()
+#' writeLines(c("x <- 1", "y <- x + 1"), tmp)
+#' get_source_expressions(tmp)
+#' unlink(tmp)
 #' @export
-#' @md
 get_source_expressions <- function(filename, lines = NULL) {
   source_expression <- srcfile(filename, encoding = settings$encoding)
 
@@ -67,8 +74,7 @@ get_source_expressions <- function(filename, lines = NULL) {
   }
 
   # Only regard explicit attribute terminal_newline=FALSE as FALSE and all other cases (e.g. NULL or TRUE) as TRUE.
-  # We don't use isFALSE since it is introduced in R 3.5.0.
-  terminal_newline <- !identical(attr(source_expression$lines, "terminal_newline", exact = TRUE), FALSE)
+  terminal_newline <- !isFALSE(attr(source_expression$lines, "terminal_newline", exact = TRUE))
 
   e <- NULL
   source_expression$lines <- extract_r_source(
@@ -80,7 +86,7 @@ get_source_expressions <- function(filename, lines = NULL) {
   source_expression$content <- get_content(source_expression$lines)
   parsed_content <- get_source_expression(source_expression, error = function(e) lint_parse_error(e, source_expression))
 
-  if (inherits(e, "lint") && (is.na(e$line) || !nzchar(e$line))) {
+  if (is_lint(e) && (is.na(e$line) || !nzchar(e$line) || e$message == "unexpected end of input")) {
     # Don't create expression list if it's unreliable (invalid encoding or unhandled parse error)
     expressions <- list()
   } else {
@@ -98,24 +104,25 @@ get_source_expressions <- function(filename, lines = NULL) {
 
     if (!is.null(xml_parsed_content) && !is.na(xml_parsed_content)) {
       expression_xmls <- lapply(
-        xml2::xml_find_all(xml_parsed_content, "/exprlist/*"),
+        xml_find_all(xml_parsed_content, "/exprlist/*"),
         function(top_level_expr) xml2::xml_add_parent(xml2::xml_new_root(top_level_expr), "exprlist")
       )
       for (i in seq_along(expressions)) {
         expressions[[i]]$xml_parsed_content <- expression_xmls[[i]]
+        expressions[[i]]$xml_find_function_calls <- build_xml_find_function_calls(expression_xmls[[i]])
       }
     }
 
     # add global expression
-    expressions[[length(expressions) + 1L]] <-
-      list(
-        filename = filename,
-        file_lines = source_expression$lines,
-        content = source_expression$lines,
-        full_parsed_content = parsed_content,
-        full_xml_parsed_content = xml_parsed_content,
-        terminal_newline = terminal_newline
-      )
+    expressions[[length(expressions) + 1L]] <- list(
+      filename = filename,
+      file_lines = source_expression$lines,
+      content = source_expression$lines,
+      full_parsed_content = parsed_content,
+      full_xml_parsed_content = xml_parsed_content,
+      xml_find_function_calls = build_xml_find_function_calls(xml_parsed_content),
+      terminal_newline = terminal_newline
+    )
   }
 
   list(expressions = expressions, error = e, lines = source_expression$lines)
@@ -150,6 +157,30 @@ lint_parse_error <- function(e, source_expression) {
   lint_parse_error_nonstandard(e, source_expression)
 }
 
+#' Ensure a string is valid for printing
+#'
+#' Helper to ensure a valid string is provided as line where necessary.
+#' Handles two cases:
+#'
+#'  1. NA lines, as generated by running outside of code blocks in Rmd documents
+#'  2. invalid string lines, as generated by invalid encoding settings
+#'
+#' [nchar()] can detect both cases: 1. returns `NA_integer_`, 2. errors
+#'
+#' @param line Possibly misencoded or NA line
+#'
+#' @return `line` if it's a valid non-NA string, otherwise an empty string.
+#'
+#' @noRd
+fixup_line <- function(line) {
+  nchars <- tryCatch(nchar(line, type = "chars"), error = function(e) NA_integer_)
+  if (is.na(nchars)) {
+    ""
+  } else {
+    line
+  }
+}
+
 #' Convert an R >= 4.3.0 classed parseError with metadata into a lint
 #'
 #' @param e A parse error of class `parseError` generated by R >= 4.3.0
@@ -159,8 +190,8 @@ lint_parse_error <- function(e, source_expression) {
 #'
 #' @noRd
 lint_parse_error_r43 <- function(e, source_expression) {
-  msg <- rex::re_substitutes(e$message, rex::rex(" (", except_some_of(")"), ")", end), "")
-  line <- e$lineno
+  msg <- re_substitutes(e$message, rex(" (", except_some_of(")"), ")", end), "")
+  line_number <- e$lineno
   column <- e$colno
   substr(msg, 1L, 1L) <- toupper(substr(msg, 1L, 1L))
   msg <- paste0(msg, ".")
@@ -170,23 +201,27 @@ lint_parse_error_r43 <- function(e, source_expression) {
   }
 
   if (column == 0L) {
-    line <- line - 1L
-    column <- nchar(source_expression$lines[[line]])
+    line_number <- line_number - 1L
   }
 
-  if (line < 1L || line > length(source_expression$lines)) {
+  if (line_number < 1L || line_number > length(source_expression$lines)) {
     # Safely handle invalid location info
-    line <- 1L
+    line_number <- 1L
     column <- 1L
   }
 
+  line <- fixup_line(source_expression$lines[[line_number]])
+  if (column == 0L || column > nchar(line) + 1L) {
+    column <- nchar(line) + 1L
+  }
+
   Lint(
     filename = source_expression$filename,
-    line_number = line,
+    line_number = line_number,
     column_number = column,
     type = "error",
     message = msg,
-    line = source_expression$lines[[line]]
+    line = line
   )
 }
 
@@ -206,10 +241,10 @@ lint_parse_error_r42 <- function(message_info, source_expression) {
   # end of the previous line
   if (column_number %==% 0L) {
     line_number <- line_number - 1L
-    line <- source_expression$lines[[line_number]]
-    column_number <- nchar(line)
+    line <- fixup_line(source_expression$lines[[line_number]])
+    column_number <- nchar(line) + 1L
   } else {
-    line <- source_expression$lines[[line_number]]
+    line <- fixup_line(source_expression$lines[[line_number]])
   }
 
   Lint(
@@ -235,6 +270,7 @@ lint_parse_error_r42 <- function(message_info, source_expression) {
 #' @noRd
 lint_parse_error_nonstandard <- function(e, source_expression) {
   if (grepl("invalid multibyte character in parser at line", e$message, fixed = TRUE)) {
+    # nocov start: platform-specific
     l <- as.integer(re_matches(
       e$message,
       rex("invalid multibyte character in parser at line ", capture(name = "line", digits))
@@ -247,9 +283,10 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
         column_number = 1L,
         type = "error",
         message = "Invalid multibyte character in parser. Is the encoding correct?",
-        line = source_expression$lines[[l]]
+        line = fixup_line(source_expression$lines[[l]])
       )
     )
+    # nocov end
   } else if (grepl("invalid multibyte string, element", e$message, fixed = TRUE)) {
     # Invalid encoding, will break even re_matches() below, so we need to handle this first.
     return(
@@ -263,12 +300,14 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
       )
     )
   } else if (grepl("repeated formal argument", e$message, fixed = TRUE)) {
+    # nocov start: only throws on certain versions; tested on L389
     matches <- re_matches(
       e$message,
-      rex("repeated formal argument '",
-          capture(name = "symbol", anything),
-          "' on line ",
-          capture(name = "line", digits)
+      rex(
+        "repeated formal argument '",
+        capture(name = "symbol", anything),
+        "' on line ",
+        capture(name = "line", digits)
       )
     )
     sym <- matches$symbol
@@ -281,9 +320,10 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
         column_number = 1L,
         type = "error",
         message = sprintf("Repeated formal argument '%s'.", sym),
-        line = source_expression$lines[[l]]
+        line = fixup_line(source_expression$lines[[l]])
       )
     )
+    # nocov end
   }
 
   # Hand-crafted regex to parse all error messages generated by the R parser code for R < 4.3.
@@ -294,8 +334,8 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
   # parser_files <- c("src/main/gram.c", "src/main/character.c")
   #
   # lines <- unlist(lapply(
-  #   parser_files,
-  #   function(f) readLines(paste0("https://raw.githubusercontent.com/wch/r-source/trunk/", f))
+  #   file.path("https://raw.githubusercontent.com/wch/r-source/trunk", parser_files),
+  #   readLines
   # ))
   # error_calls <- grep("error(_(", lines, fixed = TRUE, value = TRUE)
   # error_formats <- trimws(gsub("^.*error\\(_\\(\"(.+)\".+", "\\1", error_calls))
@@ -313,6 +353,7 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
   )
 
   if (grepl(parse_error_rx, e$message, perl = TRUE)) {
+    # nocov start: only throws on certain versions. Tested on L396: Nul character not allowed
     rx_match <- re_matches(
       e$message,
       parse_error_rx
@@ -334,12 +375,17 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
         line = source_expression$lines[[l]]
       )
     )
+    # nocov end
   }
 
-  message_info <- re_matches(e$message,
-                             rex(single_quotes, capture(name = "name", anything), single_quotes,
-                                 anything,
-                                 double_quotes, capture(name = "starting", anything), double_quotes))
+  message_info <- re_matches(
+    e$message,
+    rex(
+      single_quotes, capture(name = "name", anything), single_quotes,
+      anything,
+      double_quotes, capture(name = "starting", anything), double_quotes
+    )
+  )
 
   loc <- re_matches(source_expression$content, rex(message_info$starting), locations = TRUE)
   line_location <- loc[!is.na(loc$start) & !is.na(loc$end), ]
@@ -347,12 +393,16 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
   if (nrow(line_location) == 0L) {
     if (grepl("attempt to use zero-length variable name", e$message, fixed = TRUE)) {
       # empty symbol: ``, ``(), ''(), ""(), fun(''=42), fun(""=42), fun(a=1,""=42)
-      loc <- re_matches(source_expression$content,
-                        rex("``" %or%
-                              list(or("''", '""'), any_spaces, "(") %or%
-                              list(or("(", ","), any_spaces, or("''", '""'), any_spaces, "=")),
-                        options = "multi-line",
-                        locations = TRUE)
+      loc <- re_matches(
+        source_expression$content,
+        rex(
+          "``" %or%
+            list(or("''", '""'), any_spaces, "(") %or%
+            list(or("(", ","), any_spaces, or("''", '""'), any_spaces, "=")
+        ),
+        options = "multi-line",
+        locations = TRUE
+      )
       loc <- loc[!is.na(loc$start) & !is.na(loc$end), ]
       if (nrow(loc) > 0L) {
         line_location <- loc[1L, ]
@@ -388,20 +438,20 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
 }
 
 lint_rmd_error <- function(e, source_expression) {
-  message_info <- re_matches(e$message,
-    rex(except_some_of(":"),
+  message_info <- re_matches(
+    e$message,
+    rex(
+      except_some_of(":"),
       ":",
-      capture(name = "line",
-        digits),
+      capture(name = "line", digits),
       ":",
-      capture(name = "column",
-        digits),
+      capture(name = "column", digits),
       ":",
       space,
-      capture(name = "message",
-        anything),
-      "\n")
+      capture(name = "message", anything),
+      "\n"
     )
+  )
 
   line_number <- as.integer(message_info$line)
   column_number <- as.integer(message_info$column)
@@ -427,7 +477,6 @@ get_single_source_expression <- function(loc,
   content <- get_content(source_expression$lines, parsed_content[loc, ])
   id <- parsed_content$id[loc]
   pc <- parsed_content[which(top_level_map == id), ]
-  newline_locs <- get_newline_locs(content)
   list(
     filename = filename,
     line = parsed_content[loc, "line1"],
@@ -435,17 +484,17 @@ get_single_source_expression <- function(loc,
     lines = expr_lines,
     parsed_content = pc,
     xml_parsed_content = xml2::xml_missing(),
-    content = content,
-    find_line = find_line_fun(content, newline_locs),
-    find_column = find_column_fun(content, newline_locs)
+    # Placeholder for xml_find_function_calls, if needed (e.g. on R <= 4.0.5 with input source "\\")
+    xml_find_function_calls = build_xml_find_function_calls(xml2::xml_missing()),
+    content = content
   )
 }
 
 get_source_expression <- function(source_expression, error = identity) {
   parse_error <- FALSE
 
-  tryCatch(
-    source_expression$parsed_content <- parse(
+  parsed_content <- tryCatch(
+    parse(
       text = source_expression$content,
       srcfile = source_expression,
       keep.source = TRUE
@@ -453,70 +502,40 @@ get_source_expression <- function(source_expression, error = identity) {
     error = error
   )
 
-  # This needs to be done twice to avoid
-  #   https://bugs.r-project.org/bugzilla/show_bug.cgi?id=16041
-  e <- tryCatch(
-    source_expression$parsed_content <- parse(
-      text = source_expression$content,
-      srcfile = source_expression,
-      keep.source = TRUE
-    ),
-    error = error
-  )
-
-  if (inherits(e, c("error", "lint"))) {
-    assign("e", e, envir = parent.frame())
+  if (is_error(parsed_content) || is_lint(parsed_content)) {
+    assign("e", parsed_content, envir = parent.frame())
     parse_error <- TRUE
   }
 
   # Triggers an error if the lines contain invalid characters.
-  e <- tryCatch(
+  parsed_content <- tryCatch(
     nchar(source_expression$content, type = "chars"),
     error = error
   )
 
-  if (inherits(e, c("error", "lint"))) {
+  if (is_error(parsed_content) || is_lint(parsed_content)) {
     # Let parse errors take precedence over encoding problems
-    if (!parse_error) assign("e", e, envir = parent.frame())
+    if (!parse_error) assign("e", parsed_content, envir = parent.frame())
     return() # parsed_content is unreliable if encoding is invalid
   }
 
+  source_expression$parsed_content <- parsed_content
   fix_octal_escapes(fix_eq_assigns(fix_tab_indentations(source_expression)), source_expression$lines)
 }
 
 get_newline_locs <- function(x) {
   newline_search <- re_matches(x, rex("\n"), locations = TRUE, global = TRUE)[[1L]]$start
-  c(0L,
+  c(
+    0L,
     if (!is.na(newline_search[1L])) newline_search,
     nchar(x) + 1L
   )
 }
 
-find_line_fun <- function(content, newline_locs) {
-  function(line_number) {
-    warning(
-      "find_line is deprecated and will soon be removed. ",
-      "XPath logic and xml_nodes_to_lints() are usually preferable"
-    )
-    which(newline_locs >= line_number)[1L] - 1L
-  }
-}
-
-find_column_fun <- function(content, newline_locs) {
-  function(line_number) {
-    warning(
-      "find_column is deprecated and will soon be removed. ",
-      "XPath logic and xml_nodes_to_lints() are usually preferable"
-    )
-    matched_line_number <- which(newline_locs >= line_number)[1L] - 1L
-    line_number - newline_locs[matched_line_number]
-  }
-}
-
 # Fix column numbers when there are tabs
 # getParseData() counts 1 tab as a variable number of spaces instead of one:
 # https://github.com/wch/r-source/blame/e7401b68ab0e032fce3e376aaca9a5431619b2b4/src/main/gram.y#L512
-# The number of spaces is so that the code is brought to the next 8-character indentation level e.g:
+# The number of spaces is so that the code is brought to the next 8-character indentation level e.g.:
 #   "1\t;"          --> "1       ;"
 #   "12\t;"         --> "12      ;"
 #   "123\t;"        --> "123     ;"
@@ -575,7 +594,8 @@ tab_offsets <- function(tab_columns) {
   vapply(
     tab_columns - 1L,
     function(tab_idx) {
-      offset <- 7L - (tab_idx + cum_offset) %% 8L  # using a tab width of 8 characters
+      # nolint next: object_overwrite_linter. 'offset' is a perfect name here.
+      offset <- 7L - (tab_idx + cum_offset) %% 8L # using a tab width of 8 characters
       cum_offset <<- cum_offset + offset
       offset
     },
@@ -593,7 +613,7 @@ fix_eq_assigns <- function(pc) {
 
   eq_assign_locs <- which(pc$token == "EQ_ASSIGN")
   # check whether the equal-assignment is the final entry
-  if (length(eq_assign_locs) == 0L || utils::tail(eq_assign_locs, 1L) == nrow(pc)) {
+  if (length(eq_assign_locs) == 0L || tail(eq_assign_locs, 1L) == nrow(pc)) {
     return(pc)
   }
 
@@ -616,32 +636,23 @@ fix_eq_assigns <- function(pc) {
     parent = integer(n_expr),
     token = character(n_expr),
     terminal = logical(n_expr),
-    text = character(n_expr),
-    stringsAsFactors = FALSE
+    text = character(n_expr)
   )
 
   for (i in seq_len(n_expr)) {
-    start <- true_locs[i]
+    start_loc <- true_locs[i]
+    end_loc <- true_locs[i]
 
-    # TODO(michaelchirico): vectorize this loop away. the tricky part is,
-    #   this loop doesn't execute on most R versions (we tried 3.6.3 and 4.2.0).
-    #   so it likely requires some GHA print debugging -- tedious :)
-    end <- true_locs[i]
-    j <- end + 1L
-    while (j <= length(expr_locs) && !expr_locs[j]) {
-      end <- j
-      j <- j + 1L
-    }
-
-    prev_loc <- prev_locs[start]
-    next_loc <- next_locs[end]
+    prev_loc <- prev_locs[start_loc]
+    next_loc <- next_locs[end_loc]
 
+    id_itr <- id_itr + 1L
     supplemental_content[i, ] <- list(
       pc[prev_loc, "line1"],
       pc[prev_loc, "col1"],
       pc[next_loc, "line2"],
       pc[next_loc, "col2"],
-      id_itr <- id_itr + 1L,
+      id_itr,
       pc[eq_assign_locs[true_locs[i]], "parent"],
       "expr", # R now uses "equal_assign"
       FALSE,
@@ -649,9 +660,9 @@ fix_eq_assigns <- function(pc) {
     )
 
     new_parent_locs <- c(
-      prev_locs[start:end],
-      eq_assign_locs[start:end],
-      next_locs[start:end],
+      prev_locs[start_loc:end_loc],
+      eq_assign_locs[start_loc:end_loc],
+      next_locs[start_loc:end_loc],
       next_loc
     )
     pc[new_parent_locs, "parent"] <- id_itr
