@@ -16,14 +16,26 @@
 #' @param fun For additional flexibility, a function to search for in
 #' the `token` column of `parsed_content`. Typically `==` or `%in%`.
 #' @param source_file (DEPRECATED) Same as `source_expression`. Will be removed.
+#'
+#' @examples
+#' tmp <- tempfile()
+#' writeLines(c("x <- 1", "y <- x + 1"), tmp)
+#' source_exprs <- get_source_expressions(tmp)
+#' ids_with_token(source_exprs$expressions[[1L]], value = "SYMBOL")
+#' with_id(source_exprs$expressions[[1L]], 2L)
+#' unlink(tmp)
+#'
 #' @return `ids_with_token`: The indices of the `parsed_content` data frame
 #' entry of the list of source expressions. Indices correspond to the
 #' *rows* where `fun` evaluates to `TRUE` for the `value` in the *token* column.
 #' @export
-ids_with_token <- function(source_expression, value, fun = `==`, source_file) {
+ids_with_token <- function(source_expression, value, fun = `==`, source_file = NULL) {
   if (!missing(source_file)) {
-    lintr_deprecated(old = "source_file", new = "source_expression", version = "3.0.0", type = "Argument")
-    source_expression <- source_file
+    lintr_deprecated(
+      what = "source_file", alternative = "source_expression",
+      version = "3.0.0", type = "Argument",
+      signal = "stop"
+    )
   }
   if (!is_lint_level(source_expression, "expression")) {
     return(integer())
