@@ -105,12 +105,9 @@ infix_spaces_linter <- function(exclude_operators = NULL, allow_multiple_spaces
     )
   ]")
 
-  Linter(function(source_expression) {
-    if (!is_lint_level(source_expression, "expression")) {
-      return(list())
-    }
-
+  Linter(linter_level = "expression", function(source_expression) {
     xml <- source_expression$xml_parsed_content
+
     bad_expr <- xml_find_all(xml, xpath)
 
     xml_nodes_to_lints(
