@@ -1,63 +1,49 @@
-#' Lintr
-#'
-#' Checks adherence to a given style, syntax errors and possible semantic issues.
-#' Supports on the fly checking of R code edited with Emacs, Vim and Sublime Text.
-#' @seealso [lint()], [lint_package()], [lint_dir()], [linters]
-#' @importFrom stats na.omit
-#' @importFrom utils capture.output getParseData relist
-#' @keywords internal
-"_PACKAGE"
-
 #' Lint a file, directory, or package
 #'
 #' * `lint()` lints a single file.
 #' * `lint_dir()` lints all files in a directory.
 #' * `lint_package()` lints all likely locations for R files in a package, i.e.
-#'   `R/`, `tests/`, `inst/`, `vignettes/`, `data-raw/`, and `demo/`.
+#'   `R/`, `tests/`, `inst/`, `vignettes/`, `data-raw/`, `demo/`, and `exec/`.
 #'
 #' Read `vignette("lintr")` to learn how to configure which linters are run
 #' by default.
 #' Note that if files contain unparseable encoding problems, only the encoding problem will be linted to avoid
 #' unintelligible error messages from other linters.
 #'
-#' @param filename either the filename for a file to lint, or a character string of inline R code for linting.
-#' The latter (inline data) applies whenever `filename` has a newline character (\\n).
-#' @param linters a named list of linter functions to apply. See [linters] for a full list of default and available
-#' linters.
+#' @param filename Either the filename for a file to lint, or a character string of inline R code for linting.
+#'   The latter (inline data) applies whenever `filename` has a newline character (\\n).
+#' @param linters A named list of linter functions to apply. See [linters] for a full list of default and available
+#'   linters.
 #' @param ... Provide additional arguments to be passed to:
-#' - [exclude()] (in case of `lint()`; e.g. `lints` or `exclusions`)
-#' - [lint()] (in case of `lint_dir()` and `lint_package()`; e.g. `linters` or `cache`)
-#' @param cache given a logical, toggle caching of lint results. If passed a character string, store the cache in this
-#' directory.
-#' @param parse_settings whether to try and parse the settings.
+#'   - [exclude()] (in case of `lint()`; e.g. `lints` or `exclusions`)
+#'   - [lint()] (in case of `lint_dir()` and `lint_package()`; e.g. `linters` or `cache`)
+#' @param cache When logical, toggle caching of lint results. If passed a character string, store the cache in this
+#'   directory.
+#' @param parse_settings Logical, default `TRUE`. Whether to try and parse the [settings][read_settings]. Otherwise,
+#'   the [default_settings()] are used.
 #' @param text Optional argument for supplying a string or lines directly, e.g. if the file is already in memory or
-#' linting is being done ad hoc.
+#'   linting is being done ad hoc.
 #'
-#' @aliases lint_file
-# TODO(next release after 3.0.0): remove the alias
-#' @return A list of lint objects.
+#' @return An object of class `c("lints", "list")`, each element of which is a `"list"` object.
 #'
 #' @examples
-#' \dontrun{
-#'   lint("some/file-name.R") # linting a file
-#'   lint("a = 123\n")        # linting inline-code
-#'   lint(text = "a = 123")   # linting inline-code
-#' }
+#' # linting inline-code
+#' lint("a = 123\n")
+#' lint(text = "a = 123")
+#'
+#' # linting a file
+#' f <- tempfile()
+#' writeLines("a=1", f)
+#' lint(f)
+#' unlink(f)
 #'
 #' @export
 lint <- function(filename, linters = NULL, ..., cache = FALSE, parse_settings = TRUE, text = NULL) {
-  # TODO(next release after 3.0.0): remove this deprecated workaround
-  dots <- list(...)
-  if (has_positional_logical(dots)) {
-    warning(
-      "'cache' is no longer available as a positional argument; please supply 'cache' as a named argument instead. ",
-      "This warning will be upgraded to an error in the next release."
-    )
-    cache <- dots[[1L]]
-    dots <- dots[-1L]
-  }
+  # TODO(#2502): Remove this workaround.
+  dot_names <- if (getRversion() %in% c("4.1.1", "4.1.2")) names(list(...)) else ...names()
+  check_dots(dot_names, c("exclude", "parse_exclusions"))
 
-  needs_tempfile <- missing(filename) || rex::re_matches(filename, rex::rex(newline))
+  needs_tempfile <- missing(filename) || re_matches(filename, rex(newline))
   inline_data <- !is.null(text) || needs_tempfile
   lines <- get_lines(filename, text)
 
@@ -69,12 +55,12 @@ lint <- function(filename, linters = NULL, ..., cache = FALSE, parse_settings =
     close(con)
   }
 
-  filename <- normalizePath(filename, mustWork = !inline_data)  # to ensure a unique file in cache
+  filename <- normalize_path(filename, mustWork = !inline_data) # to ensure a unique file in cache
   source_expressions <- get_source_expressions(filename, lines)
 
   if (isTRUE(parse_settings)) {
     read_settings(filename)
-    on.exit(clear_settings, add = TRUE)
+    on.exit(reset_settings(), add = TRUE)
   }
 
   linters <- define_linters(linters)
@@ -86,29 +72,43 @@ lint <- function(filename, linters = NULL, ..., cache = FALSE, parse_settings =
   lint_obj <- define_cache_key(filename, inline_data, lines)
   lints <- retrieve_file(lint_cache, lint_obj, linters)
   if (!is.null(lints)) {
-    # TODO: once cache= is fully deprecated as 3rd positional argument (see top of body), we can restore the cleaner:
-    # > exclude(lints, lines = lines, ...)
-    return(do.call(exclude, c(list(lints, lines = lines, linter_names = names(linters)), dots)))
+    return(exclude(lints, lines = lines, linter_names = names(linters), ...))
   }
 
+  file_linter_names <- names(linters)[vapply(linters, is_linter_level, logical(1L), "file")]
+  expression_linter_names <- names(linters)[vapply(linters, is_linter_level, logical(1L), "expression")]
+
   lints <- list()
   if (!is_tainted(source_expressions$lines)) {
     for (expr in source_expressions$expressions) {
-      for (linter in names(linters)) {
-        lints[[length(lints) + 1L]] <- get_lints(expr, linter, linters[[linter]], lint_cache, source_expressions$lines)
+      if (is_lint_level(expr, "expression")) {
+        necessary_linters <- expression_linter_names
+      } else {
+        necessary_linters <- file_linter_names
+      }
+      for (linter in necessary_linters) {
+        # use withCallingHandlers for friendlier failures on unexpected linter errors
+        lints[[length(lints) + 1L]] <- withCallingHandlers(
+          get_lints(expr, linter, linters[[linter]], lint_cache, source_expressions$lines),
+          error = function(cond) {
+            cli_abort(
+              "Linter {.fn linter} failed in {.file {filename}}:",
+              parent = cond
+            )
+          }
+        )
       }
     }
   }
 
   lints <- maybe_append_error_lint(lints, source_expressions$error, lint_cache, filename)
-  lints <- structure(reorder_lints(flatten_lints(lints)), class = "lints")
+  lints <- reorder_lints(flatten_lints(lints))
+  class(lints) <- c("lints", "list")
 
   cache_file(lint_cache, filename, linters, lints)
   save_cache(lint_cache, filename, cache_path)
 
-  # TODO: once cache= is fully deprecated as 3rd positional argument (see top of body), we can restore the cleaner:
-  # > exclude(lints, lines = lines, ...)
-  res <- do.call(exclude, c(list(lints, lines = lines, linter_names = names(linters)), dots))
+  res <- exclude(lints, lines = lines, linter_names = names(linters), ...)
 
   # simplify filename if inline
   zap_temp_filename(res, needs_tempfile)
@@ -119,14 +119,23 @@ lint <- function(filename, linters = NULL, ..., cache = FALSE, parse_settings =
 #' @param relative_path if `TRUE`, file paths are printed using their path relative to the base directory.
 #'   If `FALSE`, use the full absolute path.
 #' @param exclusions exclusions for [exclude()], relative to the package path.
-#' @param pattern pattern for files, by default it will take files with any of the extensions .R, .Rmd, .Rnw, .Rhtml,
-#' .Rrst, .Rtex, .Rtxt allowing for lowercase r (.r, ...)
+#' @param pattern pattern for files, by default it will take files with any of the extensions
+#' .R, .Rmd, .qmd, .Rnw, .Rhtml, .Rrst, .Rtex, .Rtxt allowing for lowercase r (.r, ...).
+#' @param show_progress Logical controlling whether to show linting progress with a simple text
+#'   progress bar _via_ [utils::txtProgressBar()]. The default behavior is to show progress in
+#'   [interactive()] sessions not running a testthat suite.
+#'
 #' @examples
-#' \dontrun{
+#' if (FALSE) {
 #'   lint_dir()
+#'
 #'   lint_dir(
-#'     linters = list(semicolon_linter())
-#'     exclusions = list("inst/doc/creating_linters.R" = 1, "inst/example/bad.R", "renv")
+#'     linters = list(semicolon_linter()),
+#'     exclusions = list(
+#'       "inst/doc/creating_linters.R" = 1,
+#'       "inst/example/bad.R",
+#'       "renv"
+#'     )
 #'   )
 #' }
 #' @export
@@ -134,36 +143,33 @@ lint <- function(filename, linters = NULL, ..., cache = FALSE, parse_settings =
 lint_dir <- function(path = ".", ...,
                      relative_path = TRUE,
                      exclusions = list("renv", "packrat"),
-                     pattern = rex::rex(".", one_of("Rr"), or("", "html", "md", "nw", "rst", "tex", "txt"), end),
-                     parse_settings = TRUE) {
-  # TODO(next release after 3.0.0): remove this deprecated workaround
-  dots <- list(...)
-  if (has_positional_logical(dots)) {
-    warning(
-      "'relative_path' is no longer available as a positional argument; ",
-      "please supply 'relative_path' as a named argument instead. ",
-      "This warning will be upgraded to an error in the next release."
-    )
-    relative_path <- dots[[1L]]
-    dots <- dots[-1L]
-  }
+                     # TODO(r-lib/rex#85): Re-write in case-sensitive rex()
+                     pattern = "(?i)[.](r|rmd|qmd|rnw|rhtml|rrst|rtex|rtxt)$",
+                     parse_settings = TRUE,
+                     show_progress = NULL) {
+  # TODO(#2502): Remove this workaround.
+  dot_names <- if (getRversion() %in% c("4.1.1", "4.1.2")) names(list(...)) else ...names()
+  check_dots(dot_names, c("lint", "exclude", "parse_exclusions"))
 
   if (isTRUE(parse_settings)) {
     read_settings(path)
-    on.exit(clear_settings, add = TRUE)
+    on.exit(reset_settings(), add = TRUE)
 
     exclusions <- c(exclusions, settings$exclusions)
   }
 
+  if (is.null(show_progress)) show_progress <- interactive() && !identical(Sys.getenv("TESTTHAT"), "true")
+
   exclusions <- normalize_exclusions(
     exclusions,
     root = path,
     pattern = pattern
   )
 
-  # normalizePath ensures names(exclusions) and files have the same names for the same files.
+  # normalize_path ensures names(exclusions) and files have the same names for the same files.
+  # It also ensures all paths have forward slash
   # Otherwise on windows, files might incorrectly not be excluded in to_exclude
-  files <- normalizePath(dir(
+  files <- normalize_path(dir(
     path,
     pattern = pattern,
     recursive = TRUE,
@@ -173,22 +179,34 @@ lint_dir <- function(path = ".", ...,
   # Remove fully ignored files to avoid reading & parsing
   files <- drop_excluded(files, exclusions)
 
-  lints <- flatten_lints(lapply(
-    files,
-    function(file) {
-      maybe_report_progress()
-      # TODO: once relative_path= is fully deprecated as 2nd positional argument (see top of body), restore the cleaner:
-      # > lint(file, ..., parse_settings = FALSE, exclusions = exclusions)
-      do.call(lint, c(list(file, parse_settings = FALSE, exclusions = exclusions), dots))
-    }
-  ))
+  if (length(files) == 0L) {
+    lints <- list()
+    class(lints) <- "lints"
+    return(lints)
+  }
 
-  maybe_report_progress(done = TRUE)
+  if (isTRUE(show_progress)) {
+    lints <- lapply(
+      # NB: This cli API is experimental (https://github.com/r-lib/cli/issues/709)
+      cli::cli_progress_along(files, name = "Running linters"),
+      function(idx) {
+        lint(files[idx], ..., parse_settings = FALSE, exclusions = exclusions)
+      }
+    )
+  } else {
+    lints <- lapply(
+      files,
+      function(file) { # nolint: unnecessary_lambda_linter.
+        lint(file, ..., parse_settings = FALSE, exclusions = exclusions)
+      }
+    )
+  }
 
+  lints <- flatten_lints(lints)
   lints <- reorder_lints(lints)
 
-  if (relative_path == TRUE) {
-    path <- normalizePath(path, mustWork = FALSE)
+  if (relative_path) {
+    path <- normalize_path(path, mustWork = FALSE)
     lints[] <- lapply(
       lints,
       function(x) {
@@ -214,11 +232,11 @@ drop_excluded <- function(files, exclusions) {
 }
 
 #' @examples
-#' \dontrun{
+#' if (FALSE) {
 #'   lint_package()
 #'
 #'   lint_package(
-#'     linters = linters_with_defaults(semicolon_linter = semicolon_linter())
+#'     linters = linters_with_defaults(semicolon_linter = semicolon_linter()),
 #'     exclusions = list("inst/doc/creating_linters.R" = 1, "inst/example/bad.R")
 #'   )
 #' }
@@ -227,29 +245,26 @@ drop_excluded <- function(files, exclusions) {
 lint_package <- function(path = ".", ...,
                          relative_path = TRUE,
                          exclusions = list("R/RcppExports.R"),
-                         parse_settings = TRUE) {
-  # TODO(next release after 3.0.0): remove this deprecated workaround
-  dots <- list(...)
-  if (has_positional_logical(dots)) {
-    warning(
-      "'relative_path' is no longer available as a positional argument; ",
-      "please supply 'relative_path' as a named argument instead. ",
-      "This warning will be upgraded to an error in the next release."
-    )
-    relative_path <- dots[[1L]]
-    dots <- dots[-1L]
+                         parse_settings = TRUE,
+                         show_progress = NULL) {
+  if (length(path) > 1L) {
+    cli_abort(c(
+      x = "Only linting one package at a time is supported.",
+      i = "Instead, {.val {length(path)}} package paths were provided."
+    ))
   }
-
   pkg_path <- find_package(path)
 
   if (is.null(pkg_path)) {
-    warning("Didn't find any R package searching upwards from '", path, "'.")
+    cli_warn(c(
+      i = "Didn't find any R package searching upwards from {.file {normalize_path(path)}}"
+    ))
     return(NULL)
   }
 
   if (parse_settings) {
     read_settings(pkg_path)
-    on.exit(clear_settings, add = TRUE)
+    on.exit(reset_settings(), add = TRUE)
   }
 
   exclusions <- normalize_exclusions(
@@ -257,16 +272,17 @@ lint_package <- function(path = ".", ...,
     root = pkg_path
   )
 
-  r_directories <- file.path(pkg_path, c("R", "tests", "inst", "vignettes", "data-raw", "demo"))
-  # TODO: once relative_path= is fully deprecated as 2nd positional argument (see top of body), restore the cleaner:
-  # > lints <- lint_dir(r_directories, relative_path = FALSE, exclusions = exclusions, parse_settings = FALSE, ...)
-  lints <- do.call(
-    lint_dir,
-    c(list(r_directories, relative_path = FALSE, exclusions = exclusions, parse_settings = FALSE), dots)
+  r_directories <- file.path(pkg_path, c("R", "tests", "inst", "vignettes", "data-raw", "demo", "exec"))
+  lints <- lint_dir(r_directories,
+    relative_path = FALSE,
+    exclusions = exclusions,
+    parse_settings = FALSE,
+    show_progress = show_progress,
+    ...
   )
 
   if (isTRUE(relative_path)) {
-    path <- normalizePath(pkg_path, mustWork = FALSE)
+    path <- normalize_path(pkg_path, mustWork = FALSE)
     lints[] <- lapply(
       lints,
       function(x) {
@@ -317,36 +333,38 @@ define_linters <- function(linters = NULL) {
   } else if (is_linter(linters)) {
     linters <- list(linters)
     names(linters) <- attr(linters[[1L]], "name", exact = TRUE)
-  } else if (!is.list(linters)) {
+  } else if (is.list(linters)) {
+    names(linters) <- auto_names(linters)
+  } else {
     name <- deparse(substitute(linters))
     linters <- list(linters)
     names(linters) <- name
-  } else {
-    names(linters) <- auto_names(linters)
   }
   linters
 }
 
 validate_linter_object <- function(linter, name) {
-  if (!is_linter(linter) && is.function(linter)) {
-    if (is_linter_factory(linter)) {
-      old <- "Passing linters as variables"
-      new <- "a call to the linters (see ?linters)"
-      lintr_deprecated(old = old, new = new, version = "3.0.0",
-                       type = "")
-      linter <- linter()
-    } else {
-      old <- "The use of linters of class 'function'"
-      new <- "linters classed as 'linter' (see ?Linter)"
-      lintr_deprecated(old = old, new = new, version = "3.0.0",
-                       type = "")
-      linter <- Linter(linter, name = name)
-    }
-  } else if (!is.function(linter)) {
-    stop(gettextf("Expected '%s' to be a function of class 'linter', not a %s of class '%s'",
-                  name, typeof(linter), class(linter)[[1L]]))
+  if (is_linter(linter)) {
+    return(linter)
   }
-  linter
+  if (!is.function(linter)) {
+    cli_abort(c(
+      i = "Expected {.fn {name}} to be a function of class {.cls linter}.",
+      x = "Instead, it is {.obj_type_friendly {linter}}."
+    ))
+  }
+  if (is_linter_factory(linter)) {
+    what <- "Passing linters as variables"
+    alternative <- "a call to the linters (see ?linters)"
+  } else {
+    what <- "The use of linters of class 'function'"
+    alternative <- "linters classed as 'linter' (see ?Linter)"
+  }
+  lintr_deprecated(
+    what = what, alternative = alternative, version = "3.0.0",
+    type = "",
+    signal = "stop"
+  )
 }
 
 is_linter_factory <- function(fun) {
@@ -371,73 +389,6 @@ reorder_lints <- function(lints) {
 }
 
 
-has_description <- function(path) {
-  desc_info <- file.info(file.path(path, "DESCRIPTION"))
-  !is.na(desc_info$size) && desc_info$size > 0.0 && !desc_info$isdir
-}
-
-find_package <- function(path) {
-  depth <- 2
-  while(!has_description(path)) {
-    path <- dirname(path)
-    if (is_root(path) || depth <= 0) {
-      return(NULL)
-    }
-    depth <- depth - 1
-  }
-  path
-}
-
-find_rproj_or_package <- function(path) {
-  path <- normalizePath(path, mustWork = FALSE)
-
-  depth <- 2
-  while(!(has_description(path) || has_rproj(path))) {
-    path <- dirname(path)
-    if (is_root(path) || depth <= 0) {
-      return(NULL)
-    }
-    depth <- depth - 1
-  }
-  path
-}
-
-has_rproj <- function(path) {
-  length(head(Sys.glob(file.path(path, "*.Rproj")), n = 1L)) == 1
-}
-
-find_rproj_at <- function(path) {
-  head(Sys.glob(file.path(path, "*.Rproj")), n = 1L)
-}
-is_root <- function(path) {
-  identical(path, dirname(path))
-}
-
-has_config <- function(path, config) {
-  file.exists(file.path(path, config))
-}
-
-find_config2 <- function(path) {
-  config <- basename(getOption("lintr.linter_file"))
-  path <- normalizePath(path, mustWork = FALSE)
-
-  while (!has_config(path, config)) {
-    path <- dirname(path)
-    if (is_root(path)) {
-      return(character())
-    }
-  }
-  return(file.path(path, config))
-}
-
-pkg_name <- function(path = find_package()) {
-  if (is.null(path)) {
-    return(NULL)
-  } else {
-    read.dcf(file.path(path, "DESCRIPTION"), fields = "Package")[1L]
-  }
-}
-
 #' Create a `lint` object
 #' @param filename path to the source file that was linted.
 #' @param line_number line number where the lint occurred.
@@ -447,7 +398,7 @@ pkg_name <- function(path = find_package()) {
 #' @param line code source where the lint occurred
 #' @param ranges a list of ranges on the line that should be emphasized.
 #' @param linter deprecated. No longer used.
-#' @return an object of class 'lint'.
+#' @return an object of class `c("lint", "list")`.
 #' @name lint-s3
 #' @export
 Lint <- function(filename, line_number = 1L, column_number = 1L, # nolint: object_name.
@@ -455,31 +406,83 @@ Lint <- function(filename, line_number = 1L, column_number = 1L, # nolint: objec
                  message = "", line = "", ranges = NULL, linter = "") {
   if (!missing(linter)) {
     lintr_deprecated(
-      old = "Using the `linter` argument of `Lint()`",
+      what = "Using the `linter` argument of `Lint()`",
       version = "3.0.0",
-      type = ""
+      type = "",
+      signal = "stop"
     )
   }
 
+  if (length(line) != 1L || !is.character(line)) {
+    cli_abort("{.arg line} must be a string.", call. = FALSE)
+  }
+  max_col <- max(nchar(line) + 1L, 1L, na.rm = TRUE)
+  if (!is_number(column_number) || column_number < 0L || column_number > max_col) {
+    cli_abort("
+      {.arg column_number} must be an integer between {.val {0}} and {.val {max_col}} ({.code nchar(line) + 1}),
+      not {.obj_type_friendly {column_number}}.
+    ")
+  }
+  if (!is_number(line_number) || line_number < 1L) {
+    cli_abort("{.arg line_number} must be a positive integer, not {.obj_type_friendly {line_number}}.")
+  }
+  check_ranges(ranges, max_col)
+
   type <- match.arg(type)
 
-  structure(
-    list(
-      filename = filename,
-      line_number = as.integer(line_number),
-      column_number = as.integer(column_number),
-      type = type,
-      message = message,
-      line = line,
-      ranges = ranges,
-      linter = NA_character_
-    ),
-    class = "lint")
+  obj <- list(
+    filename = filename,
+    line_number = as.integer(line_number),
+    column_number = as.integer(column_number),
+    type = type,
+    message = message, # nolint: undesirable_function_linter
+    line = line,
+    ranges = ranges,
+    linter = NA_character_
+  )
+  class(obj) <- c("lint", "list")
+  obj
+}
+
+is_number <- function(number, n = 1L) {
+  length(number) == n && is.numeric(number) && !anyNA(number)
+}
+
+is_valid_range <- function(range, max_col) {
+  0L <= range[[1L]] &&
+    range[[1L]] <= range[[2L]] &&
+    range[[2L]] <= max_col
+}
+
+check_ranges <- function(ranges, max_col, call = parent.frame()) {
+  if (is.null(ranges)) {
+    return()
+  }
+  if (!is.list(ranges)) {
+    cli_abort(
+      "{.arg ranges} must be {.code NULL} or a list, not {.obj_type_friendly {ranges}}.",
+      call = call
+    )
+  }
+
+  for (range in ranges) {
+    if (!is_number(range, 2L)) {
+      cli_abort(
+        "{.arg ranges} must only contain integer vectors of length 2 without {.code NA}s.",
+        call = call
+      )
+    } else if (!is_valid_range(range, max_col)) {
+      cli_abort(
+        "{.arg ranges} must satisfy {.val {0}} <= range[1L] <= range[2L] <= {.val {max_col}} (nchar(line) + 1).",
+        call = call
+      )
+    }
+  }
 }
 
 rstudio_source_markers <- function(lints) {
   if (!requireNamespace("rstudioapi", quietly = TRUE)) {
-    stop("'rstudioapi' is required for rstudio_source_markers().") # nocov
+    cli_abort("{.pkg rstudioapi} is required for {.fn rstudio_source_markers}.") # nocov
   }
 
   # package path will be NULL unless it is a relative path
@@ -530,7 +533,6 @@ rstudio_source_markers <- function(lints) {
 #' @param filename the name of the output report
 #' @export
 checkstyle_output <- function(lints, filename = "lintr_results.xml") {
-
   # package path will be NULL unless it is a relative path
   package_path <- attr(lints, "path")
 
@@ -552,12 +554,12 @@ checkstyle_output <- function(lints, filename = "lintr_results.xml") {
         f, "error",
         line = as.character(x$line_number),
         column = as.character(x$column_number),
-        severity = switch(
-          x$type,
+        severity = switch(x$type,
           style = "info",
           x$type
         ),
-        message = x$message)
+        message = x$message
+      )
     })
   })
 
@@ -572,70 +574,20 @@ checkstyle_output <- function(lints, filename = "lintr_results.xml") {
 #' @param filename the name of the output report
 #' @export
 sarif_output <- function(lints, filename = "lintr_results.sarif") {
+  if (!requireNamespace("jsonlite", quietly = TRUE)) {
+    cli_abort("{.pkg jsonlite} is required to produce SARIF reports. Please install to continue.") # nocov
+  }
 
-  # package path will be NULL unless it is a relative path
+  # package path will be `NULL` unless it is a relative path
   package_path <- attr(lints, "path")
 
+  if (is.null(package_path)) {
+    cli_abort("Package path needs to be a relative path.")
+  }
+
   # setup template
   sarif <- jsonlite::fromJSON(
-    '{
-      "$schema": "https://schemastore.azurewebsites.net/schemas/json/sarif-2.1.0-rtm.5.json",
-      "version": "2.1.0",
-      "runs": [
-        {
-          "tool": {
-            "driver": {
-              "name": "lintr",
-              "informationUri": "https://lintr.r-lib.org/",
-              "version": "2.0.1",
-              "rules": [
-                {
-                  "id": "trailing_whitespace_linter",
-                  "fullDescription": {
-                    "text": "Trailing whitespace is superfluous."
-                  },
-                  "defaultConfiguration": {
-                    "level": "note"
-                  }
-                }
-              ]
-            }
-          },
-          "results": [
-            {
-              "ruleId": "trailing_whitespace_linter",
-              "ruleIndex": 0,
-              "message": {
-                "text": "Trailing blank lines are superfluous."
-              },
-              "locations": [
-                {
-                  "physicalLocation": {
-                    "artifactLocation": {
-                      "uri": "TestFileFolder/hello.r",
-                      "uriBaseId": "ROOTPATH"
-                    },
-                    "region": {
-                      "startLine": 2,
-                      "startColumn": 22,
-                      "snippet": {
-                        "text": "print(Hello World!) "
-                      }
-                    }
-                  }
-                }
-              ]
-            }
-          ],
-          "columnKind": "utf16CodeUnits",
-          "originalUriBaseIds": {
-            "ROOTPATH": {
-              "uri": "file:///C:/repos/repototest/"
-            }
-          }
-        }
-      ]
-    }',
+    system.file("extdata", "sarif-template.json", package = "lintr"),
     simplifyVector = TRUE,
     simplifyDataFrame = FALSE,
     simplifyMatrix = FALSE
@@ -644,20 +596,19 @@ sarif_output <- function(lints, filename = "lintr_results.sarif") {
   # assign values
   sarif$runs[[1L]]$results <- NULL
   sarif$runs[[1L]]$tool$driver$rules <- NULL
-  sarif$runs[[1L]]$tool$driver$version <-
-    as.character(utils::packageVersion("lintr"))
+  sarif$runs[[1L]]$tool$driver$version <- as.character(utils::packageVersion("lintr"))
   sarif$runs[[1L]]$originalUriBaseIds$ROOTPATH$uri <- ""
   rule_index_exists <- FALSE
-  root_path_uri <- gsub("\\\\", "/", package_path)
+  root_path_uri <- gsub("\\", "/", package_path, fixed = TRUE)
 
   if (startsWith(root_path_uri, "/")) {
-    root_path_uri <- paste("file://", root_path_uri, sep = "")
+    root_path_uri <- paste0("file://", root_path_uri)
   } else {
-    root_path_uri <- paste("file:///", root_path_uri, sep = "")
+    root_path_uri <- paste0("file:///", root_path_uri) # nocov
   }
 
   if (!endsWith(root_path_uri, "/")) {
-    root_path_uri <- paste(root_path_uri, "/", sep = "")
+    root_path_uri <- paste0(root_path_uri, "/")
   }
 
   sarif$runs[[1L]]$originalUriBaseIds$ROOTPATH$uri <- root_path_uri
@@ -670,10 +621,12 @@ sarif_output <- function(lints, filename = "lintr_results.sarif") {
       rule_index_exists <- 0L
     } else {
       rule_index_exists <-
-        which(sapply(sarif$runs[[1L]]$tool$driver$rules,
-                     function(x) x$id == lint$linter))
-      if (length(rule_index_exists) == 0L ||
-          is.na(rule_index_exists[1L])) {
+        which(vapply(
+          sarif$runs[[1L]]$tool$driver$rules,
+          function(x) x$id == lint$linter,
+          logical(1L)
+        ))
+      if (length(rule_index_exists) == 0L || is.na(rule_index_exists[1L])) {
         rule_index_exists <- 0L
       }
     }
@@ -682,12 +635,11 @@ sarif_output <- function(lints, filename = "lintr_results.sarif") {
       new_rule <- list(
         id = lint$linter,
         fullDescription = list(text = lint$message),
-        defaultConfiguration = list(level = switch(lint$type,
-                                                   style = "note",
-                                                   lint$type))
+        defaultConfiguration = list(
+          level = switch(lint$type, style = "note", lint$type)
+        )
       )
-      sarif$runs[[1L]]$tool$driver$rules <-
-        append(sarif$runs[[1L]]$tool$driver$rules, list(new_rule))
+      sarif$runs[[1L]]$tool$driver$rules <- append(sarif$runs[[1L]]$tool$driver$rules, list(new_rule))
       rule_index <- length(sarif$runs[[1L]]$tool$driver$rules) - 1L
     } else {
       rule_index <- rule_index_exists - 1L
@@ -695,11 +647,10 @@ sarif_output <- function(lints, filename = "lintr_results.sarif") {
 
     one_result <- append(one_result, c(ruleId = lint$linter))
     one_result <- append(one_result, c(ruleIndex = rule_index))
-    one_result <-
-      append(one_result, list(message = list(text = lint$message)))
+    one_result <- append(one_result, list(message = list(text = lint$message)))
     one_location <- list(physicalLocation = list(
       artifactLocation = list(
-        uri = gsub("\\\\", "/", lint$filename),
+        uri = gsub("\\", "/", lint$filename, fixed = TRUE),
         uriBaseId = "ROOTPATH"
       ),
       region = list(
@@ -708,27 +659,26 @@ sarif_output <- function(lints, filename = "lintr_results.sarif") {
         snippet = list(text = lint$line)
       )
     ))
-    one_result <-
-      append(one_result, c(locations = list(list(one_location))))
-
-    sarif$runs[[1L]]$results <-
-      append(sarif$runs[[1L]]$results, list(one_result))
+    one_result <- append(one_result, c(locations = list(list(one_location))))
+    sarif$runs[[1L]]$results <- append(sarif$runs[[1L]]$results, list(one_result))
   }
 
-  write(jsonlite::toJSON(sarif, pretty = TRUE, auto_unbox = TRUE),
-        filename)
+  # if lints is empty, add empty results list
+  if (length(lints) == 0L) {
+    sarif$runs[[1L]]$results <- list()
+  }
+
+  write(jsonlite::toJSON(sarif, pretty = TRUE, auto_unbox = TRUE), filename)
 }
 
 highlight_string <- function(message, column_number = NULL, ranges = NULL) {
-
   maximum <- max(column_number, unlist(ranges))
 
   line <- fill_with(" ", maximum)
 
-  lapply(ranges, function(range) {
-    substr(line, range[1L], range[2L]) <<-
-      fill_with("~", range[2L] - range[1L] + 1L)
-  })
+  for (range in ranges) {
+    substr(line, range[1L], range[2L]) <- fill_with("~", range[2L] - range[1L] + 1L)
+  }
 
   substr(line, column_number, column_number + 1L) <- "^"
 
@@ -736,7 +686,7 @@ highlight_string <- function(message, column_number = NULL, ranges = NULL) {
 }
 
 fill_with <- function(character = " ", length = 1L) {
-  paste0(collapse = "", rep.int(character, length))
+  paste(collapse = "", rep.int(character, length))
 }
 
 has_positional_logical <- function(dots) {
@@ -745,20 +695,8 @@ has_positional_logical <- function(dots) {
     !nzchar(names2(dots)[1L])
 }
 
-maybe_report_progress <- function(done = FALSE) {
-  if (interactive() && !identical(Sys.getenv("TESTTHAT"), "true")) {
-    # nocov start
-    if (done) {
-      message()
-    } else {
-      message(".", appendLF = FALSE)
-    }
-    # nocov end
-  }
-}
-
 maybe_append_error_lint <- function(lints, error, lint_cache, filename) {
-  if (inherits(error, "lint")) {
+  if (is_lint(error)) {
     error$linter <- "error"
     lints[[length(lints) + 1L]] <- error
 
@@ -772,7 +710,7 @@ maybe_append_error_lint <- function(lints, error, lint_cache, filename) {
 get_lines <- function(filename, text) {
   if (!is.null(text)) {
     strsplit(paste(text, collapse = "\n"), "\n", fixed = TRUE)[[1L]]
-  } else if (rex::re_matches(filename, rex::rex(newline))) {
+  } else if (re_matches(filename, rex(newline))) {
     strsplit(gsub("\n$", "", filename), "\n", fixed = TRUE)[[1L]]
   } else {
     read_lines(filename)
