@@ -39,14 +39,10 @@
 #' @seealso [linters] for a complete list of linters available in lintr.
 #' @export
 namespace_linter <- function(check_exports = TRUE, check_nonexports = TRUE) {
-  Linter(function(source_expression) {
-    if (!is_lint_level(source_expression, "file")) {
-      return(list())
-    }
-
+  Linter(linter_level = "file", function(source_expression) {
     xml <- source_expression$full_xml_parsed_content
 
-    ns_nodes <- xml2::xml_find_all(xml, "//NS_GET | //NS_GET_INT")
+    ns_nodes <- xml_find_all(xml, "//NS_GET | //NS_GET_INT")
 
     if (length(ns_nodes) == 0L) {
       return(list())
@@ -54,7 +50,7 @@ namespace_linter <- function(check_exports = TRUE, check_nonexports = TRUE) {
 
     ## Case 1: pkg is uninstalled in pkg::foo
 
-    package_nodes <- xml2::xml_find_all(ns_nodes, "preceding-sibling::*[1]")
+    package_nodes <- xml_find_all(ns_nodes, "preceding-sibling::*[1]")
     packages <- get_r_string(package_nodes)
 
     lints <- list()
@@ -88,15 +84,12 @@ namespace_linter <- function(check_exports = TRUE, check_nonexports = TRUE) {
 
     # nocov start
     if (any(failed_namespace)) {
-      stop(
-        "Failed to retrieve namespaces for one or more of the packages used with `::` or `:::`. ",
-        "Please report the issue at https://github.com/r-lib/lintr/issues."
-      )
+      cli_abort_internal("Failed to retrieve namespaces for one or more of the packages used with `::` or `:::`. ")
     }
     # nocov end
 
-    ns_get <- xml2::xml_text(ns_nodes) == "::"
-    symbol_nodes <- xml2::xml_find_all(ns_nodes, "following-sibling::*[1]")
+    ns_get <- xml_text(ns_nodes) == "::"
+    symbol_nodes <- xml_find_all(ns_nodes, "following-sibling::*[1]")
     symbols <- get_r_string(symbol_nodes)
 
     if (check_nonexports) {
@@ -123,74 +116,59 @@ namespace_linter <- function(check_exports = TRUE, check_nonexports = TRUE) {
   })
 }
 
-get_all_exports <- function(namespace) {
-  c(getNamespaceExports(namespace), names(.getNamespaceInfo(namespace, "lazydata")))
+namespace_symbols <- function(ns, exported = TRUE) {
+  if (exported) {
+    c(getNamespaceExports(ns), names(.getNamespaceInfo(ns, "lazydata")))
+  } else {
+    ls(ns, all.names = TRUE)
+  }
+}
+is_in_pkg <- function(symbols, namespaces, exported = TRUE) {
+  vapply(
+    seq_along(symbols),
+    function(ii) symbols[[ii]] %in% namespace_symbols(namespaces[[ii]], exported = exported),
+    logical(1L)
+  )
 }
 
 build_ns_get_int_lints <- function(packages, symbols, symbol_nodes, namespaces, source_expression) {
-  lints <- list()
-
   ## Case 2: foo does not exist in pkg:::foo
-
-  non_symbols <- !vapply(
-    seq_along(symbols),
-    function(ii) symbols[[ii]] %in% ls(namespaces[[ii]], all.names = TRUE),
-    logical(1L)
+  non_symbols <- !is_in_pkg(symbols, namespaces, exported = FALSE)
+  non_symbols_lints <- xml_nodes_to_lints(
+    symbol_nodes[non_symbols],
+    source_expression = source_expression,
+    lint_message = sprintf("'%s' does not exist in {%s}.", symbols[non_symbols], packages[non_symbols]),
+    type = "warning"
   )
-  if (any(non_symbols)) {
-    lints <- c(lints, xml_nodes_to_lints(
-      symbol_nodes[non_symbols],
-      source_expression = source_expression,
-      lint_message = sprintf("'%s' does not exist in {%s}.", symbols[non_symbols], packages[non_symbols]),
-      type = "warning"
-    ))
 
-    packages <- packages[!non_symbols]
-    symbols <- symbols[!non_symbols]
-    symbol_nodes <- symbol_nodes[!non_symbols]
-  }
+  packages <- packages[!non_symbols]
+  symbols <- symbols[!non_symbols]
+  symbol_nodes <- symbol_nodes[!non_symbols]
+  namespaces <- namespaces[!non_symbols]
 
   ## Case 3: foo is already exported pkg:::foo
-
-  exported <- vapply(
-    seq_along(symbols),
-    function(ii) symbols[[ii]] %in% get_all_exports(namespaces[[ii]]),
-    logical(1L)
+  exported <- is_in_pkg(symbols, namespaces)
+  exported_lints <- xml_nodes_to_lints(
+    symbol_nodes[exported],
+    source_expression = source_expression,
+    lint_message =
+      sprintf("Don't use `:::` to access %s, which is exported from %s.", symbols[exported], packages[exported]),
+    type = "warning"
   )
-  if (any(exported)) {
-    lints <- c(lints, xml_nodes_to_lints(
-      symbol_nodes[exported],
-      source_expression = source_expression,
-      lint_message =
-        sprintf("'%1$s' is exported from {%2$s}. Use %2$s::%1$s instead.", symbols[exported], packages[exported]),
-      type = "warning"
-    ))
-  }
 
-  lints
+  c(non_symbols_lints, exported_lints)
 }
 
 build_ns_get_lints <- function(packages, symbols, symbol_nodes, namespaces, source_expression) {
-  lints <- list()
-
   # strip backticked symbols; `%>%` is the same as %>% (#1752).
   symbols <- gsub("^`(.*)`$", "\\1", symbols)
 
   ## Case 4: foo is not an export in pkg::foo
-
-  unexported <- !vapply(
-    seq_along(symbols),
-    function(ii) symbols[[ii]] %in% get_all_exports(namespaces[[ii]]),
-    logical(1L)
+  unexported <- !is_in_pkg(symbols, namespaces)
+  xml_nodes_to_lints(
+    symbol_nodes[unexported],
+    source_expression = source_expression,
+    lint_message = sprintf("'%s' is not exported from {%s}.", symbols[unexported], packages[unexported]),
+    type = "warning"
   )
-  if (any(unexported)) {
-    lints <- c(lints, xml_nodes_to_lints(
-      symbol_nodes[unexported],
-      source_expression = source_expression,
-      lint_message = sprintf("'%s' is not exported from {%s}.", symbols[unexported], packages[unexported]),
-      type = "warning"
-    ))
-  }
-
-  lints
 }
