@@ -2,22 +2,25 @@
 #'
 #' Check that there are no trailing blank lines in source code.
 #'
-#' @examplesIf requireNamespace("withr", quietly = TRUE)
+#' @examples
 #' # will produce lints
-#' f <- withr::local_tempfile(lines = "x <- 1\n")
-#' readLines(f)
+#' f <- tempfile()
+#' cat("x <- 1\n\n", file = f)
+#' writeLines(readChar(f, file.size(f)))
 #' lint(
 #'   filename = f,
 #'   linters = trailing_blank_lines_linter()
 #' )
+#' unlink(f)
 #'
 #' # okay
-#' f <- withr::local_tempfile(lines = "x <- 1")
-#' readLines(f)
+#' cat("x <- 1\n", file = f)
+#' writeLines(readChar(f, file.size(f)))
 #' lint(
 #'   filename = f,
 #'   linters = trailing_blank_lines_linter()
 #' )
+#' unlink(f)
 #'
 #' @evalRd rd_tags("trailing_blank_lines_linter")
 #' @seealso [linters] for a complete list of linters available in lintr.
@@ -38,7 +41,7 @@ trailing_blank_lines_linter <- function() {
           line_number = line_number,
           column_number = 1L,
           type = "style",
-          message = "Trailing blank lines are superfluous.",
+          message = "Remove trailing blank lines.",
           line = source_expression$file_lines[[line_number]]
         )
       }
@@ -51,9 +54,9 @@ trailing_blank_lines_linter <- function() {
       lints[[length(lints) + 1L]] <- Lint(
         filename = source_expression$filename,
         line_number = length(source_expression$file_lines),
-        column_number = nchar(last_line) %||% 0L + 1L,
+        column_number = (nchar(last_line) %||% 0L) + 1L,
         type = "style",
-        message = "Missing terminal newline.",
+        message = "Add a terminal newline.",
         line = last_line
       )
     }
