@@ -34,10 +34,10 @@ fhash <- function(filename) {
 # `clear_cache`
 
 test_that("clear_cache deletes the file if a file is given", {
-  skip_if_not_installed("mockery")
-
-  mockery::stub(clear_cache, "read_settings", function(...) invisible(...))
-  mockery::stub(clear_cache, "unlink", function(...) list(...))
+  local_mocked_bindings(
+    read_settings = function(...) invisible(...),
+    unlink = function(...) list(...)
+  )
 
   e1 <- new.env(parent = emptyenv())
   d1 <- withr::local_tempfile(pattern = "lintr_cache_")
@@ -50,10 +50,10 @@ test_that("clear_cache deletes the file if a file is given", {
 })
 
 test_that("clear_cache deletes the directory if no file is given", {
-  skip_if_not_installed("mockery")
-
-  mockery::stub(clear_cache, "read_settings", function(...) invisible(...))
-  mockery::stub(clear_cache, "unlink", function(...) list(...))
+  local_mocked_bindings(
+    read_settings = function(...) invisible(...),
+    unlink = function(...) list(...)
+  )
 
   expect_identical(clear_cache(file = NULL, path = "."), list(".", recursive = TRUE))
 })
@@ -421,10 +421,9 @@ test_that("lint with cache uses the provided relative cache directory", {
 })
 
 test_that("it works outside of a package", {
-  skip_if_not_installed("mockery")
   linter <- assignment_linter()
 
-  mockery::stub(lintr:::find_default_encoding, "find_package", function(...) NULL)
+  local_mocked_bindings(find_package = function(...) NULL)
   path <- withr::local_tempfile(pattern = "my_cache_dir_")
   expect_false(dir.exists(path))
   expect_lint("a <- 1", NULL, linter, cache = path)
@@ -436,15 +435,16 @@ test_that("it works outside of a package", {
 
 test_that("cache = TRUE workflow works", {
   # Need a test structure with a safe to load .lintr
-  pkg <- "dummy_packages/package"
-  files <- normalizePath(list.files(pkg, recursive = TRUE, full.names = TRUE))
+  withr::local_dir(file.path("dummy_packages", "package"))
+  withr::local_options(lintr.linter_file = "lintr_test_config")
+  files <- normalize_path(list.files(recursive = TRUE, full.names = TRUE))
 
   # Manually clear cache (that function is exported)
   for (f in files) {
     clear_cache(file = f)
   }
-  l1 <- lint_package(pkg, cache = TRUE)
-  l2 <- lint_package(pkg, cache = TRUE)
+  l1 <- lint_package(cache = TRUE)
+  l2 <- lint_package(cache = TRUE)
   expect_identical(l1, l2)
 })
 
