@@ -19,3 +19,19 @@ test_that("expect_not_linter blocks simple disallowed usages", {
   expect_lint("expect_false(!foo(x))", lint_msg, linter)
   expect_lint("testthat::expect_true(!(x && y))", lint_msg, linter)
 })
+
+test_that("lints vectorize", {
+  lint_msg <- rex::rex("expect_false(x) is better than expect_true(!x), and vice versa.")
+
+  expect_lint(
+    trim_some("{
+      expect_true(!x)
+      expect_false(!y)
+    }"),
+    list(
+      list(lint_msg, line_number = 2L),
+      list(lint_msg, line_number = 3L)
+    ),
+    expect_not_linter()
+  )
+})
