@@ -4,7 +4,7 @@ test_that("marginfigure engine from tufte package doesn't cause problems", {
 
   expect_lint(
     file = test_path("knitr_extended_formats", "tufte.Rmd"),
-    checks = list(rex::rex("Use <-, not =, for assignment."), line_number = 11L),
+    checks = list(rex::rex("Use one of <-, <<- for assignment, not =."), line_number = 11L),
     default_linters,
     parse_settings = FALSE
   )
@@ -16,7 +16,7 @@ test_that("engines from bookdown package cause no problems", {
 
   expect_lint(
     file = test_path("knitr_extended_formats", "bookdown.Rmd"),
-    checks = list(rex::rex("Use <-, not =, for assignment."), line_number = 14L),
+    checks = list(rex::rex("Use one of <-, <<- for assignment, not =."), line_number = 14L),
     default_linters,
     parse_settings = FALSE
   )
