@@ -211,7 +211,7 @@ test_that("package using .lintr.R config lints correctly", {
   # config has bad R syntax
   expect_error(
     lint_package(test_path("dummy_packages", "RConfigInvalid")),
-    "Malformed config file, ensure it is valid R syntax",
+    "Malformed config file (lintr_test_config.R), ensure it is valid R syntax",
     fixed = TRUE
   )
 
@@ -219,7 +219,7 @@ test_that("package using .lintr.R config lints correctly", {
   withr::local_options(lintr.linter_file = "lintr_test_config_extraneous")
   expect_warning(
     expect_length(lint_package(r_config_pkg), 2L),
-    "Found unused settings in config",
+    "Found unused settings in config file",
     fixed = TRUE
   )
 
@@ -231,7 +231,6 @@ test_that("package using .lintr.R config lints correctly", {
 })
 
 test_that("lintr need not be attached for .lintr.R configs to use lintr functions", {
-  skip_if_not_r_version("3.6.0") # unclear error message
   exprs <- paste(
     'options(lintr.linter_file = "lintr_test_config")',
     sprintf('lints <- lintr::lint_package("%s")', test_path("dummy_packages", "RConfig")),
