@@ -1,23 +1,19 @@
 test_that("input validation for available_linters works as expected", {
-  expect_error(available_linters(1L), "`packages` must be a character vector.")
-  expect_error(available_linters(tags = 1L), "`tags` must be a character vector.")
-  expect_error(available_linters(exclude_tags = 1L), "`exclude_tags` must be a character vector.")
+  expect_error(available_linters(1L), "`packages` must be a <character> vector.")
+  expect_error(available_linters(tags = 1L), "`tags` must be a <character> vector.")
+  expect_error(available_linters(exclude_tags = 1L), "`exclude_tags` must be a <character> vector.")
 })
 
 test_that("validate_linter_db works as expected", {
   df_empty <- data.frame()
   expect_warning(
     lintr:::validate_linter_db(df_empty, "mypkg"),
-    "`linters.csv` must contain the columns 'linter' and 'tags'.",
+    'must contain the columns "linter" and "tags"',
     fixed = TRUE
   )
   expect_false(suppressWarnings(lintr:::validate_linter_db(df_empty, "mypkg")))
 
-  df <- data.frame(
-    linter = "absolute_path_linter",
-    tags = "robustness",
-    stringsAsFactors = FALSE
-  )
+  df <- data.frame(linter = "absolute_path_linter", tags = "robustness")
   expect_true(lintr:::validate_linter_db(df, "mypkg"))
 })
 
@@ -79,11 +75,14 @@ test_that("warnings occur only for deprecated linters", {
 
 test_that("available_linters matches the set of linters available from lintr", {
   lintr_db <- available_linters(exclude_tags = NULL)
-  linters_in_namespace <- setdiff(ls(asNamespace("lintr"), pattern = "_linter$"), "is_linter")
+  defunct_linters <-
+    subset(read.csv(system.file("lintr", "linters.csv", package = "lintr"), as.is = TRUE), tags == "defunct")$linter
+  ignore <- c("is_linter", defunct_linters)
+  linters_in_namespace <- setdiff(ls(asNamespace("lintr"), pattern = "_linter$"), ignore)
   # ensure that the contents of inst/lintr/linters.csv covers all _linter objects in our namespace
   expect_identical(sort(lintr_db$linter), sort(linters_in_namespace))
   # ensure that all _linter objects in our namespace are also exported
-  exported_linters <- setdiff(grep("_linter$", getNamespaceExports("lintr"), value = TRUE), "is_linter")
+  exported_linters <- setdiff(grep("_linter$", getNamespaceExports("lintr"), value = TRUE), ignore)
   expect_identical(sort(linters_in_namespace), sort(exported_linters))
 })
 
@@ -104,12 +103,7 @@ test_that("rownames for available_linters data frame doesn't have missing entrie
 # See the roxygen helpers in R/linter_tags.R for the code used to generate the docs.
 #   This test helps ensure the documentation is up to date with the available_linters() database
 test_that("lintr help files are up to date", {
-  help_db <- tools::Rd_db("lintr")
-  # e.g. in dev under pkgload::load_all()
-  if (length(help_db) == 0L) {
-    help_db <- tools::Rd_db(dir = test_path("..", ".."))
-    skip_if_not(length(help_db) > 0L, message = "Package help not installed or corrupted")
-  }
+  help_db <- safe_load_help_db()
 
   lintr_db <- available_linters(exclude_tags = NULL)
   lintr_db$package <- NULL
@@ -157,6 +151,7 @@ test_that("lintr help files are up to date", {
   )
 
   # Counts of tags from available_linters()
+  #   NB: as.data.frame.table returns stringsAsFactors=TRUE default in R>4
   db_tag_table <- as.data.frame(
     table(tag = unlist(lintr_db$tags)),
     responseName = "n_linters",
@@ -234,3 +229,7 @@ test_that("available_linters gives precedence to included tags", {
     available_linters(tags = "deprecated", exclude_tags = NULL)
   )
 })
+
+test_that("all linters have at least one tag", {
+  expect_true(all(lengths(available_linters()$tags) > 0L))
+})
