@@ -1,11 +1,18 @@
 test_that("modify_defaults produces error with missing or incorrect defaults", {
-  lint_msg <- "`defaults` must be a named list."
-  expect_error(modify_defaults(), lint_msg, fixed = TRUE)
-  expect_error(modify_defaults("assignment_linter"), lint_msg, fixed = TRUE)
+  expect_error(
+    modify_defaults(),
+    "`defaults` is a required argument, but is missing",
+    fixed = TRUE
+  )
+  expect_error(
+    modify_defaults("assignment_linter"),
+    "`defaults` must be a named list",
+    fixed = TRUE
+  )
 })
 
 test_that("linters_with_tags produces error with incorrect tags", {
-  expect_error(linters_with_tags(1L:4L), "`tags` must be a character vector, or NULL.", fixed = TRUE)
+  expect_error(linters_with_tags(1L:4L), "`tags` must be a character vector, or `NULL`", fixed = TRUE)
 })
 
 test_that("linters_with_defaults works as expected with unnamed args", {
@@ -22,22 +29,21 @@ test_that("linters_with_defaults warns on unused NULLs", {
 })
 
 test_that("linters_with_tags() verifies the output of available_linters()", {
-  skip_if_not_installed("mockery")
-  mockery::stub(
-    linters_with_tags,
-    "available_linters",
-    data.frame(linter = c("fake_linter", "very_fake_linter"), package = "lintr", tags = "", stringsAsFactors = FALSE)
+  local_mocked_bindings(
+    available_linters = function(...) {
+      data.frame(linter = c("fake_linter", "very_fake_linter"), package = "lintr", tags = "")
+    }
   )
   expect_error(
     linters_with_tags(NULL),
-    "'fake_linter' and 'very_fake_linter'"
+    "Can't find linters `fake_linter()` and `very_fake_linter()`",
+    fixed = TRUE
   )
 })
 
 test_that("all default linters are tagged default", {
   expect_named(linters_with_defaults(), available_linters(tags = "default")$linter)
 
-  skip_if_not_installed("waldo", "0.4.0") # needs waldo#133
   # covr modifies package functions causing differing deparse() results even for identical anonymous functions.
   # This happens because default_linters is generated at build time and thus not modifiable by covr, whereas
   # linters_with_tags() constructs the linters at runtime.
@@ -68,25 +74,11 @@ test_that("can instantiate all linters without arguments", {
   expect_length(really_all_linters, nrow(available_linters(exclude_tags = NULL)))
 })
 
-test_that("with_defaults is supported with a deprecation warning", {
-  defaults <- linters_with_defaults()
-  expect_warning(
-    {
-      old_defaults <- with_defaults()
-    },
+test_that("with_defaults is fully deprecated", {
+  expect_error(
+    with_defaults(),
     rex::rex("Use linters_with_defaults or modify_defaults instead.")
   )
-  expect_identical(defaults, old_defaults)
-
-  # linters_with_defaults only accepts `defaults = list()` to start from blank
-  defaults <- linters_with_defaults(defaults = list(), whitespace_linter())
-  expect_warning(
-    {
-      old_defaults <- with_defaults(default = NULL, whitespace_linter())
-    },
-    rex::rex("Use linters_with_defaults or modify_defaults instead.")
-  )
-  expect_identical(defaults, old_defaults)
 })
 
 test_that("modify_defaults works", {
@@ -104,7 +96,7 @@ test_that("linters_with_defaults(default = .) is supported with a deprecation wa
     {
       linters <- linters_with_defaults(default = list(), whitespace_linter())
     },
-    "'default'"
+    "`default` is not an argument"
   )
   expect_named(linters, "whitespace_linter")
 
