@@ -8,14 +8,11 @@ test_that("with_id works as expected", {
   expect_identical(ref$token, rep_len("expr", nrow(ref)))
 
   # deprecated argument
-  expect_warning(
-    {
-      old_arg <- with_id(
-        source_file = source_expression,
-        id = ids_with_token(source_expression = source_expression, value = "expr")
-      )
-    },
+  expect_error(
+    with_id(
+      source_file = source_expression,
+      id = ids_with_token(source_expression = source_expression, value = "expr")
+    ),
     "Argument source_file was deprecated"
   )
-  expect_identical(old_arg, ref)
 })
